import argparse
from collections import namedtuple, Mapping
import csv
import sys

import arrow


def namedtuple_with_defaults(typename, field_names, default_values=()):
    T = namedtuple(typename, field_names)
    T.__new__.__defaults__ = (None,) * len(T._fields)
    if isinstance(default_values, Mapping):
        prototype = T(**default_values)
    else:
        prototype = T(*default_values)
    T.__new__.__defaults__ = tuple(prototype)
    return T


# story, topic: Your current work, where 'story' is more general than 'topic'.
field_names = ['date', 'type', 'desc', 'story', 'duration', 'status',
               'details']
event = namedtuple_with_defaults('event', field_names,
                                 default_values={'status': None,
                                                 'details': None})


def get_monday():
    today = arrow.now()
    monday = today.replace(days=-today.weekday())
    # If today is Thursday or before, we probably want *last* monday.
    # Because what sort of crazy person would be doing this horrible thing
    # more than once a week?
    if today.weekday() <= 3:
        monday = monday.replace(weeks=-1)
    return monday


def write_events(events):
    # Sort events by time, for ease of inspection and editing.
    events = sorted(events, key=lambda event: event.date)
    # Write to standard-out.
    csv_writer = csv.DictWriter(sys.stdout, fieldnames=field_names)
    csv_writer.writeheader()
    for event in events:
        csv_writer.writerow(event._asdict())


def valid_monday(s):
    """Convert a string to a date, and check it is a Monday.."""
    d = arrow.get(s)
    if d.weekday() > 0:
        raise argparse.ArgumentTypeError('Not a monday')
    return d


def monday_to_week_days(monday):
    week_days = [monday.replace(days=+i) for i in range(5)]
    return (day.format('YYYY-MM-DD') for day in week_days)



from os.path import dirname, basename, splitext
import sys


def make_module_path_importable(file_path):
    dir_name = dirname(file_path)
    sys.path.append(dir_name)


def file_path_to_module_name(file_path):
    return splitext(basename(file_path))[0]



import argparse
import importlib

from . import wbso_lib, utils


description = '''
usage examples:
  gowbso wbso_data
  gowbso ~/admin/wbso/wbso_data.py
  gowbso --monday 2017-02-27 wbso_data
'''


def main():
    parser = argparse.ArgumentParser(
        description='Make WBSO CSV files quickly',
        epilog=description,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument(
        'events_module',
        help='A Python module containing the week\'s activities. '
             'May be either the name of a Python module that can be imported, '
             'or a path to a Python file.'
    )
    parser.add_argument(
        '-m', '--monday', type=wbso_lib.valid_monday,
        help='A string representing the Monday that begins '
             'the week for which you want to generate data'
    )
    args = parser.parse_args()

    # If the week to make data for is not specified, use the most recent
    # Monday.
    if args.monday is None:
        args.monday = wbso_lib.get_monday()

    # If passed a path to a Python file.
    if args.events_module.endswith('.py'):
        # Allow modules in that directory to be imported.
        utils.make_module_path_importable(args.events_module)
        # And get the module name for that file.
        module_name = utils.file_path_to_module_name(args.events_module)
    # Otherwise, assume we have been passed an actual module name.
    else:
        module_name = args.events_module
    # Either way, import the module.
    events_module = importlib.import_module(module_name)

    week_days = wbso_lib.monday_to_week_days(args.monday)
    events = events_module.get_events(*week_days)
    wbso_lib.write_events(events)


if __name__ == '__main__':
    main()
